@@ -1,62 +1,32 @@
-# a.rb
-# -pv-
+# a.rb $Revision: 1.7 $
 #
-# ̾Ρ
-# 󥫡ưץ饰
+# Create anchor easily.
 #
-# ס
-# ˴Ť󥫡ưޤ
-# ޤϡե˴Ť󥫡ưޤ(tDiary-1.4.1ʹ)
+# 1. Usage
+# a(url, name)
 #
-# Ȥꡧ
-# ʸإå⤷ϥեå
+#   a "http://www.hoge.com/diary/", "Hoge Diary"
 #
-# Ȥ
-# 1. a(key, option_or_name = "", name = nil)
-#	  key - 󥫡Υ̾(פʤϤ켫ȤURLˤʤ)
-#	  option_or_name - ŪɲäǤդʸʾάǽ
-#									 פʤɽʸˤʤ
-#    name - ɽʸ󡣼եȤϰ㤦̾դȤ˻Ȥʾάǽ
+# a(key, option_or_name = "", name = nil)
+#   Use dictionary file. You need to create the dictionary 
+#   file with a CGI.
+#   a "home"
+#   a "home", "20020329.html", "Here"
 #
-# a "home"
-# a "home", "20020329.html", ""
-# a "http://www.hoge.com/diary/", "Hoge Diary"
+# a("name|key:option")
+#   Use dictionary file. You need to create the dictionary 
+#   file with a CGI.
 #
-# 2. a("name|key:option")
-#    1.1ĤʸˤޤȤ᤿ǤϿΤǴ
-#    Ȥ䤹Ǥkeyoptionδ֤':'namekeyδ֤'|'Ƕڤ
-#    ޤnameoptionϾάǽǤ
-#    ޤkeyˤϥɤURLޤYYYYMM...Ȥ
-#    myȤۤƱư򤷤ޤ(titleɽǤޤ)
-#    () 1.ˡnameΰ֤ۤʤΤǺ𤷤ʤ褦ˤƤ
+#   a "key"
+#   a "key:20020329.html"
+#   a "key:20020329.html|Here"
+#   a "Hoge Diary|http://www.hoge.com/diary/"
+#   a "Hoge Diary|20020201.html#p01"  #=> Same as "my" plugin
 #
-# a "key"
-# a "key:20020329.html"
-# a "key:20020329.html|"
-# a "Hoge Diary|http://www.hoge.com/diary/"
-# a "Hoge Diary|20020201.html#p01"  #=> myƱǤ(ϵդ)
-#
-# ¾Υץ:
-# @options['a.tlink'] = true
-#  true˻ꤹȡtitle°ͤȤƥξɽޤ
-#  Υץͭˤ硢󡢥ؤΥ
-#  ȯޤ󥿥륵ǻѤ٤⤯ʤäƤޤä
-#  falseˤƤꤷʤfalseǤ
-#
-# @options['a.path'] = "/home/hoge/"
-#  եȤϡեpathꤷޤ
-#  ꤷʤϡ@data_path/cache/a.datˤʤޤ
-#
-# եԽCGIƤӽФܥ:
-# եԽCGIƤӽФܥꤷޤإå⤷ϥեå֤
-# 
-#
-# navi_a(name = "a.rb") - եԽCGIƤӽФޤ
-#	name - ܥ̾Ρʾάϡ"a.rb"ˤʤ
-#
-# ¾
-# ¾ξ http://home2.highway.ne.jp/mutoh/tools/ruby/ja/a.html 
-# 򻲾ȤƤ
+# 4. Documents
+# See URLs below for more details.
+#   http://ponx.s5.xrea.com/hiki/a.rb.html (English) 
+#   http://ponx.s5.xrea.com/hiki/ja/a.rb.html (Japanese) 
 # 
 # Copyright (c) 2002,2003 MUTOH Masao <mutoh@highway.ne.jp>
 # You can redistribute it and/or modify it under GPL2.
@@ -171,12 +141,12 @@ def a(key, option_or_name = nil, name = nil, charset = nil)
 			result = "tlink is not available."
 		end
 	else
-		result = %Q[<a href="#{url}">#{value}</a>]
+		result = %Q[<a href="#{CGI.escapeHTML(url)}">#{value}</a>]
 	end
 	result
 end
 
-def navi_a(name = "a.rb")
+def navi_a(name = "a.rb conf")
 	"<span class=\"adminmenu\"><a href=\"a_conf.rb\">#{name}</a></span>\n"
 end
 
