@@ -1,4 +1,4 @@
-# image.rb $Revision: 1.19 $
+# image.rb $Revision: 1.22 $
 # -pv-
 # 
 # ̾:
@@ -72,7 +72,7 @@ unless @resource_loaded then
 	def image_label_add_plugin; 'ʸɲ'; end
 	def image_label_delete; 'åκ'; end
 	def image_label_only_jpeg; 'JPEGΤ'; end
-	def image_label_add_image; 'ɲ'; end
+	def image_label_add_image; 'β򥢥åץɤ'; end
 end
 
 def image( id, alt = 'image', thumbnail = nil, size = nil, place = 'photo' )
@@ -266,10 +266,11 @@ add_form_proc do |date|
 		//-->
 		</script>
 		]
-		if @conf.style == "Wiki"
+		case @conf.style.sub( /^blog/i, '' )
+		when /^wiki$/i
 			ptag1 = "{{"
 			ptag2 = "}}"
-		elsif @conf.style == "RD"
+		when /^rd$/i
 			ptag1 = "((%"
 			ptag2 = "%))"
 		else
@@ -291,8 +292,8 @@ add_form_proc do |date|
 			else
 				img_type, img_w, img_h = open(File.join(@image_dir,img).untaint, 'r') {|f| image_info(f)}
 			end
-			r << %Q[<td><img class="form" src="#{@image_url}/#{img}" alt="#{id}" width="#{(img_w && img_w > 160) ? 160 : img_w}"></td>]
-			ptag = "#{ptag1}image #{id}, '', nil, #{img_w && img_h ? '['+img_w.to_s+','+img_h.to_s+']' : 'nil'}#{ptag2}"
+			r << %Q[<td><img class="form" src="#{@image_url}/#{img}" alt="#{id}" width="#{(img_w && img_w > 160) ? 160 : (img_w ? img_w : 160)}"></td>]
+			ptag = "#{ptag1}image #{id}, '#{image_label_description}', nil, #{img_w && img_h ? '['+img_w.to_s+','+img_h.to_s+']' : 'nil'}#{ptag2}"
 			if @conf.secure then
 				img_info = ''
 			else
@@ -329,7 +330,7 @@ add_form_proc do |date|
 	#{@conf.secure ? image_label_only_jpeg : ''}
    <input type="hidden" name="plugin_image_addimage" value="true">
    <input type="hidden" name="date" value="#{date.strftime( '%Y%m%d' )}">
-   <input type="file" tabindex="#{tabidx+98}" name="plugin_image_file">
+   <input type="file" tabindex="#{tabidx+98}" name="plugin_image_file" size="50">
    <input type="submit" tabindex="#{tabidx+99}" name="plugin" value="#{image_label_add_image}">
    </div></form>
 	</div>]
