@@ -1,6 +1,6 @@
 #
 # etdiary_style.rb: tDiary style class for etDiary format.
-# $Id: etdiary_style.rb,v 1.6 2003/10/27 15:49:45 kitaj Exp $
+# $Id: etdiary_style.rb,v 1.10 2004/05/27 06:51:26 shirai-kaoru Exp $
 #
 # if you want to use this style, add @style into tdiary.conf below:
 #
@@ -115,11 +115,14 @@ module TDiary
 				end
 			end
 
-			r = "<a"
-			r << " name=\"#{name}\"" if @opt['anchor']
-			r << " href=\"" + @opt['index']
-			r << "<%=anchor \"" + date.strftime('%Y%m%d') + "#" + name + "\" %>\">"
-			r << @opt['section_anchor'] + "</a>"
+			r = ''
+			if @opt['index']
+				r << "<a"
+				r << " name=\"#{name}\"" if @opt['anchor']
+				r << " href=\"" + @opt['index']
+				r << "<%=anchor \"" + date.strftime('%Y%m%d') + "#" + name + "\" %>\">"
+				r << @opt['section_anchor'] + "</a>"
+			end
 			r << "[" + fragment.author + "]" if fragment.author
 			r << fragment.categorized_subtitle if fragment.subtitle
 
@@ -215,9 +218,9 @@ module TDiary
 	class EtdiaryDiary
 		include DiaryBase
 		include CategorizableDiary
-	
-		TAG_BEG_REGEXP = /\A<([A-Za-z]+)([^>]*)>([^\r]*)\z/
-		TAG_END_REGEXP = /\A([^\r]*)<\/([A-Za-z]+)>\n*\z/
+		
+		TAG_BEG_REGEXP = /\A<([A-Za-z][0-9A-Za-z]*)([^>]*)>([^\r]*)\z/
+		TAG_END_REGEXP = /\A([^\r]*)<\/([A-Za-z][0-9A-Za-z]*)>\n*\z/
 		PRE_REGEXP     = /\A<[Pp][Rr][Ee][^>]*>([^\r]*)<\/[Pp][Rr][Ee]>\n*\z/
 		TITLE_REGEXP   = /\A<<([^\r]*?)>>[^>]/
 	
@@ -251,11 +254,9 @@ module TDiary
 					buffer << fragment
 				else
 					title = TITLE_REGEXP.match(fragment+"\n").to_a[1]
-					if title then
-						@sections << section
-						section = EtdiarySection::new( title, author )
-						fragment = fragment[ title.length + 4 .. -1 ]
-					end
+					@sections << section
+					section = EtdiarySection::new( title, author )
+					fragment = fragment[ title.length + 4 .. -1 ] if title
 					if TAG_BEG_REGEXP =~ fragment then
 						tag_kind = $1.downcase
 						if TAG_END_REGEXP =~ fragment and $2.downcase == tag_kind then
@@ -269,6 +270,9 @@ module TDiary
 					end
 				end
 			end
+      if buffer
+        section << buffer << "</#{tag_kind}>(tDiary warning: tag &lt;#{tag_kind}&gt; is not terminated.)"
+      end
 			@zerosection = @sections.shift
 			@sections << section
 			@last_modified = Time::now
