@@ -1,20 +1,25 @@
 =begin
 == NAME
 tDiary: the "tsukkomi-able" web diary system.
-tdiary.rb $Revision: 1.156 $
+tdiary.rb $Revision: 1.193 $
 
-Copyright (C) 2001-2003, TADA Tadashi <sho@spc.gr.jp>
+Copyright (C) 2001-2004, TADA Tadashi <sho@spc.gr.jp>
 You can redistribute it and/or modify it under GPL2.
 =end
 
-TDIARY_VERSION = '1.5.6'
+TDIARY_VERSION = '2.0.0'
 
 require 'cgi'
 begin
-	require 'erb'
+	require 'erb_fast'
 	ERbLight = ERB
 rescue LoadError
-	require 'erb/erbl'
+	begin
+		require 'erb'
+		ERbLight = ERB
+	rescue LoadError
+		require 'erb/erbl'
+	end
 end
 
 =begin
@@ -32,7 +37,8 @@ class String
 			gsub( r ){ $1 == $2 ? "<a href=\"#$2\">#$2</a>" : "<a href=\"mailto:#$4\">#$4</a>" }.
 			gsub( /\003/, '&gt;' ).
 			gsub( /\002/, '&lt;' ).
-			gsub( /\001/, '&nbsp;' ).
+			gsub( /^\001+/ ) { $&.gsub( /\001/, '&nbsp;' ) }.
+			gsub( /\001/, ' ' ).
 			gsub( /\t/, '&nbsp;' * 8 )
 	end
 end
@@ -62,10 +68,14 @@ require 'thread'
 module Safe
 	def safe( level = 4 )
 		result = nil
-		Thread.start {
-			$SAFE = level
+		if $SAFE < level then
+			Thread.start {
+				$SAFE = level
+				result = yield
+			}.join
+		else
 			result = yield
-		}.join
+		end
 		result
   end
   module_function :safe
@@ -121,15 +131,11 @@ module TDiary
 	
 	public
 		def add_comment( com )
-			if @comments[-1] != com then
-				@comments << com
-				if not @last_modified or @last_modified < com.date
-					@last_modified = com.date
-				end
-				com
-			else
-				nil
+			@comments << com
+			if not @last_modified or @last_modified < com.date
+				@last_modified = com.date
 			end
+			com
 		end
 	
 		def count_comments( all = false )
@@ -250,6 +256,25 @@ module TDiary
 		end
 	end
 
+	#
+	# module/class Filter
+	#
+	module Filter
+		class Filter
+			def initialize( cgi, conf )
+				@cgi, @conf = cgi, conf
+			end
+
+			def comment_filter( diary, comment )
+				true
+			end
+
+			def referer_filter( referer )
+				true
+			end
+		end
+	end
+	
 	#
 	# module DiaryBase
 	#  Base module of Diary.
@@ -334,9 +359,9 @@ module TDiary
 		end
 
 		def styled_diary_factory( date, title, body, style = 'tDiary' )
-			begin
-				eval( "#{style( style.downcase )}::new( date, title, body )" )
-			rescue SyntaxError
+			if style_class = style( style.downcase )
+				return style_class::new( date, title, body )
+			else
 				raise BadStyleError, "bad style: #{style}"
 			end
 		end
@@ -346,7 +371,7 @@ module TDiary
 			Dir::glob( "#{TDiary::PATH}/tdiary/*_style.rb" ) do |style_file|
 				require style_file.untaint
 				style = File::basename( style_file ).sub( /_style\.rb$/, '' )
-				eval( "@styles[style] = TDiary::#{style.capitalize}Diary" )
+				@styles[style] = TDiary::const_get( "#{style.capitalize}Diary" )
 			end
 		end
 
@@ -396,7 +421,7 @@ module TDiary
 			result = ERbLight::new( File::open( "#{PATH}/skel/tdiary.rconf" ){|f| f.read }.untaint ).result( binding )
 			result.untaint unless @secure
 			Safe::safe( @secure ? 4 : 1 ) do
-				eval( result )
+				eval( result, binding, "(TDiary::Config#save)", 1 )
 			end
 			File::open( "#{@data_path}tdiary.conf", 'w' ) do |o|
 				o.print result
@@ -427,17 +452,28 @@ module TDiary
 			@options2.delete( key )
 		end
 	
+		def base_url
+			return '' unless ENV['SCRIPT_NAME']
+			if ENV['HTTPS']
+				port = (ENV['SERVER_PORT'] == '443') ? '' : ':' + ENV['SERVER_PORT'].to_s
+				"https://#{ ENV['SERVER_NAME'] }#{ port }#{File::dirname(ENV['SCRIPT_NAME'])}/"
+			else
+				port = (ENV['SERVER_PORT'] == '80') ? '' : ':' + ENV['SERVER_PORT'].to_s
+				"http://#{ ENV['SERVER_NAME'] }#{ port }#{File::dirname(ENV['SCRIPT_NAME'])}/"
+			end.sub(%r|/+$|, '/')
+		end
+
 	private
 		# loading tdiary.conf in current directory
 		def load
 			@secure = true unless @secure
 			@options = {}
-			eval( File::open( "tdiary.conf" ){|f| f.read }.untaint )
+			eval( File::open( "tdiary.conf" ){|f| f.read }.untaint, binding, "(tdiary.conf)", 1 )
 
 			# language setup
 			@lang = 'ja' unless @lang
 			begin
-				instance_eval( File::open( "#{TDiary::PATH}/tdiary/lang/#{@lang}.rb" ){|f| f.read }.untaint )
+				instance_eval( File::open( "#{TDiary::PATH}/tdiary/lang/#{@lang}.rb" ){|f| f.read }.untaint, "(tdiary/lang/#{@lang}.rb)", 1 )
 			rescue Errno::ENOENT
 				@lang = 'ja'
 				retry
@@ -514,7 +550,7 @@ module TDiary
 				variables.each do |var| def_vars << "#{var} = nil\n" end
 				eval( def_vars )
 				Safe::safe( @secure ? 4 : 1 ) do
-					eval( cgi_conf )
+					eval( cgi_conf, binding, "(TDiary::Config#cgi_conf)", 1 )
 				end
 				variables.each do |var| eval "@#{var} = #{var} if #{var} != nil" end
 			rescue IOError, Errno::ENOENT
@@ -591,7 +627,7 @@ module TDiary
 					@plugin_files << plugin_file
 				end
 			rescue Exception
-				raise PluginError::new( "Plugin error in '#{File::basename( plugin_file )}'.\n#{$!}" )
+				raise PluginError::new( "Plugin error in '#{File::basename( plugin_file )}'.\n#{$!}\n#{$!.backtrace[0]}" )
 			end
 		end
 
@@ -600,13 +636,13 @@ module TDiary
 			begin
 				res_file = File::dirname( file ) + "/#{@conf.lang}/" + File::basename( file )
 				open( res_file.untaint ) do |src|
-					instance_eval( src.read.untaint )
+					instance_eval( src.read.untaint, "(plugin/#{@conf.lang}/#{File::basename( res_file )})", 1 )
 				end
 				@resource_loaded = true
 			rescue IOError, Errno::ENOENT
 			end
-			open( file.untaint ) do |src|
-				instance_eval( src.read.untaint )
+			File::open( file.untaint ) do |src|
+				instance_eval( src.read.untaint, "(plugin/#{File::basename( file )})", 1 )
 			end
 		end
 
@@ -616,7 +652,7 @@ module TDiary
 			@body_enter_procs.taint
 			@body_leave_procs.taint
 			return Safe::safe( secure ? 4 : 1 ) do
-				eval( src )
+				eval( src, binding, "(TDiary::Plugin#eval_src)", 1 )
 			end
 		end
 
@@ -737,12 +773,15 @@ module TDiary
 		end
 
 		def apply_plugin( str, remove_tag = false )
+			return '' unless str
 			r = str.dup
 			if @options['apply_plugin'] and str.index( '<%' ) then
 				r = str.untaint if $SAFE < 3
-				r = ERbLight.new( r ).result( binding )
+				Safe::safe( @conf.secure ? 4 : 1 ) do
+					r = ERbLight.new( r ).result( binding )
+				end
 			end
-			r.gsub!( /<.*?>/, '' ) if remove_tag
+			r.gsub!( /<[^"'<>]*(?:"[^"]*"[^"'<>]*|'[^']*'[^"'<>]*)*(?:>|(?=<)|$)/, '' ) if remove_tag
 			r
 		end
 
@@ -922,9 +961,9 @@ module TDiary
 			end
 		end
 	
-		def clear_cache
+		def clear_cache( target = /.*/ )
 			Dir::glob( "#{cache_path}/*.r[bh]*" ).each do |c|
-				File::delete( c.untaint )
+				File::delete( c.untaint ) if target =~ c
 			end
 		end
 	
@@ -941,8 +980,11 @@ module TDiary
 			file = date.strftime( "#{cache_path}/%Y%m.parser" )
 	
 			unless key then
-				File::delete( file )
-				File::delete( file + '~' )
+				begin
+					File::delete( file )
+					File::delete( file + '~' )
+				rescue
+				end
 				return nil
 			end
 	
@@ -964,9 +1006,12 @@ module TDiary
 					rescue PStore::Error
 					end
 				end
-			rescue ArgumentError
-				File::delete( file )
-				File::delete( file + '~' )
+			rescue
+				begin
+					File::delete( file )
+					File::delete( file + '~' )
+				rescue
+				end
 				return nil
 			end
 			obj
@@ -975,6 +1020,33 @@ module TDiary
 		def calendar
 			@years = @io.calendar unless @years
 		end
+
+		def load_filters
+			return if @filters
+
+			@filters = []
+			filter_path = @conf.filter_path || "#{PATH}/tdiary/filter"
+			Dir::glob( "#{filter_path}/*.rb" ).sort.each do |file|
+				require file.untaint
+				@filters << TDiary::Filter::const_get( "#{File::basename( file, '.rb' ).capitalize}Filter" )::new( @cgi, @conf )
+			end
+		end
+
+		def comment_filter( diary, comment )
+			load_filters unless @filters
+			@filters.each do |filter|
+				return false unless filter.comment_filter( diary, comment )
+			end
+			true
+		end
+
+		def referer_filter( referer )
+			load_filters unless @filters
+			@filters.each do |filter|
+				return false unless filter.referer_filter( referer )
+			end
+			true
+		end
 	end
 
 	#
@@ -1079,7 +1151,9 @@ module TDiary
 	protected
 		def do_eval_rhtml( prefix )
 			super
+			@plugin.instance_eval { update_proc }
 			anchor = @plugin.instance_eval( %Q[anchor "#{@diary.date.strftime('%Y%m%d')}"].untaint )
+			clear_cache( /(latest|#{@date.strftime( '%Y%m' )})/ )
 			raise ForceRedirect::new( "#{@conf.index}#{anchor}" )
 		end
 	end
@@ -1160,7 +1234,7 @@ module TDiary
 						com.show = @cgi.params[(idx += 1).to_s][0] == 'true' ? true : false;
 					end
 					self << @diary
-					clear_cache
+					clear_cache( /(latest|#{@date.strftime( '%Y%m' )})/ )
 					dirty = DIRTY_COMMENT
 				end
 				dirty
@@ -1244,7 +1318,7 @@ module TDiary
 			super
 	
 			# save referer to latest
-			if (!@conf.referer_day_only or (@cgi.params['date'][0] and @cgi.params['date'][0].length == 8)) and referer?
+			if (!@conf.referer_day_only or (@cgi.params['date'][0] and @cgi.params['date'][0].length == 8)) and referer_filter( @cgi.referer ) then
 				ym = latest_month
 				@date = ym ? Time::local( ym[0], ym[1] ) : Time::now
 				@io.transaction( @date ) do |diaries|
@@ -1267,7 +1341,7 @@ module TDiary
 			lm = Time::at( 0 )
 			@diaries.each_value do |diary|
 				lmd = diary.last_modified
-				lm = lmd if lm < lmd
+				lm = lmd if lm < lmd and diary.visible?
 			end
 			lm
 		end
@@ -1307,22 +1381,6 @@ module TDiary
 			result
 		end
 	
-		def referer?
-			if /[\x00-\x20\x7f-\xff]/ =~ @cgi.referer then
-				return false
-			elsif @conf.bot?
-				return false
-			elsif @cgi.referer and %r|^https?://|i =~ @cgi.referer
-				ref = CGI::unescape( @cgi.referer.sub( /#.*$/, '' ).sub( /\?\d{8}$/, '' ) )
-				@conf.no_referer.each do |noref|
-					return false if /#{noref}/i =~ ref
-				end
-			else
-				return false
-			end
-			return true
-		end
-	
 		def cache_enable?( prefix )
 			super and (File::mtime( "#{cache_path}/#{cache_file( prefix )}" ) > last_modified )
 		end
@@ -1356,7 +1414,7 @@ module TDiary
 					@diaries = diaries
 					dirty = DIRTY_NONE
 					@diary = self[@date]
-					if @diary and referer? then
+					if @diary and referer_filter( @cgi.referer ) then
 						@diary.add_referer( @cgi.referer )
 						dirty = DIRTY_REFERER
 					end
@@ -1392,23 +1450,25 @@ module TDiary
 			@name = @conf.to_native( @cgi.params['name'][0] )
 			@mail = @cgi.params['mail'][0]
 			@body = @conf.to_native( @cgi.params['body'][0] )
+			@comment = Comment::new( @name, @mail, @body )
+
 			dirty = DIRTY_NONE
 			@io.transaction( @date ) do |diaries|
 				@diaries = diaries
 				@diary = self[@date]
-				if @diary and not (@name.strip.empty? or @body.strip.empty?) then
-					@comment = Comment::new( @name, @mail, @body )
-					if @diary.add_comment( @comment ) then
-						dirty = DIRTY_COMMENT
-						cookie_path = File::dirname( @cgi.script_name )
-						cookie_path += '/' if cookie_path !~ /\/$/
-						@cookies << CGI::Cookie::new( {
-							'name' => 'tdiary',
-							'value' => [@name,@mail],
-							'path' => cookie_path,
-							'expires' => Time::now.gmtime + 90*24*60*60 # 90days
-						} )
-					end
+				if @diary and comment_filter( @diary, @comment ) then
+					@diary.add_comment( @comment )
+					dirty = DIRTY_COMMENT
+					cookie_path = File::dirname( @cgi.script_name )
+					cookie_path += '/' if cookie_path !~ /\/$/
+					@cookies << CGI::Cookie::new( {
+						'name' => 'tdiary',
+						'value' => [@name,@mail],
+						'path' => cookie_path,
+						'expires' => Time::now.gmtime + 90*24*60*60 # 90days
+					} )
+				else
+					@comment = nil
 				end
 				dirty
 			end
@@ -1416,7 +1476,7 @@ module TDiary
 
 		def do_eval_rhtml( prefix )
 			load_plugins
-			@plugin.instance_eval { update_proc }
+			@plugin.instance_eval { update_proc } if @comment
 			anchor = @plugin.instance_eval( %Q[anchor "#{@diary.date.strftime('%Y%m%d')}"].untaint )
 			raise ForceRedirect::new( "#{@conf.index}#{anchor}#c#{'%02d' % @diary.count_comments( true )}" )
 		end
@@ -1555,113 +1615,7 @@ module TDiary
 		attr_reader :last_modified
 		def initialize(cgi, rhtml, conf)
 			super
-			@specified = @cgi.params['category']
-			@specified.delete("ALL")
-			@categorized = {}
 			@last_modified = Time.at(0)
-			@year, @months, @quarter = parse_cgi_param
-		end
-
-		def each_day
-			@diaries.keys.sort.each do |date|
-				diary = @diaries[date]
-				next unless diary.visible?
-				yield diary
-			end
-		end
-
-		def categorize
-			each_day do |d|
-				next unless d.categorizable?
-				idx = 1
-				d.each_section do |s|
-					s.categories.each do |c|
-						if @specified.size == 0 or @specified.include?(c)
-							@categorized[c] ||= []
-							@categorized[c] << [d.date, idx, s]
-							if @last_modified < d.last_modified
-								@last_modified = d.last_modified
-							end
-						end
-					end
-					idx += 1
-				end
-			end
-		end
-
-		def parse_cgi_param
-			if @cgi.valid?('year')
-				year = @cgi.params['year'][0].to_i
-			end
-			if @cgi.valid?('month')
-				case @cgi.params['month'][0]
-				when /(\d)Q/
-					q = $1.to_i
-					months = (((q - 1) * 3 + 1)..((q - 1) * 3 + 3)).to_a
-				when /\d{1,2}/
-					month = @cgi.params['month'][0].to_i
-					if (1..12).include?(month)
-						months = [month]
-					end
-				end
-			end
-			year ||= Time.now.year
-			months ||= [Time.now.month]
-			[year, months, q]
-		end
-	end
-
-	#
-	# class TDiaryCategoryMonth
-	#  show category by month
-	#
-	class TDiaryCategoryMonth < TDiaryCategoryView
-		def initialize(cgi, rhtml, conf)
-			super
-			@months.each do |m|
-				@date = Time::local(@year, m, 1)
-				@io.transaction(@date) do |diaries|
-					@diaries = diaries
-					DIRTY_NONE
-				end
-				categorize
-			end
-		end
-	end
-
-	#
-	# class TDiaryCategoryYear
-	#  show category by year
-	#
-	class TDiaryCategoryYear < TDiaryCategoryView
-		def initialize(cgi, rhtml, conf)
-			super
-			calendar
-			@date = Time::local(@year, 1, 1)
-			@years[@year.to_s].each do |m|
-				@date = Time::local(@year, m, 1)
-				@io.transaction(@date) do |diaries|
-					@diaries = diaries
-					DIRTY_NONE
-				end
-				categorize
-			end
-		end
-	end
-
-	#
-	# class TDiaryCategoryLatest
-	#  show category in latest month
-	#
-	class TDiaryCategoryLatest < TDiaryCategoryView
-		def initialize(cgi, rhtml, conf)
-			super
-			@date = Time.now
-			@io.transaction(@date) do |diaries|
-				@diaries = diaries
-				DIRTY_NONE
-			end
-			categorize
 		end
 	end
 
@@ -1679,23 +1633,15 @@ module TDiary
 		def initialize( cgi, rhtml, conf )
 			super
 			date = ENV['REQUEST_URI'].scan(%r!/(\d{4})(\d\d)(\d\d)!)[0]
-			@date = Time::local(*date)
-		end
-
-		def referer?
-			nil
-		end
-
-		def trackback_url
-			'http://' + ENV['HTTP_HOST'] +
-				(ENV['SERVER_PORT'] == '80' ? '' : ":#{ENV['SERVER_PORT']}") +
-				ENV['REQUEST_URI']
+			if date
+				@date = Time::local(*date)
+			else
+				@date = Time::now
+			end
 		end
 
 		def diary_url
-			trackback_url.sub(/#{File::basename(ENV['SCRIPT_NAME'])}.*$/, '') +
-				@conf.index.sub(%r|^\./|, '') +
-				@plugin.instance_eval(%Q|anchor "#{@date.strftime('%Y%m%d')}"|)
+			@conf.base_url + @conf.index.sub(%r|^\./|, '') + @plugin.instance_eval(%Q|anchor "#{@date.strftime('%Y%m%d')}"|)
 		end
 
 		def self.success_response
@@ -1783,6 +1729,9 @@ RSSFOOT
 			blog_name = @conf.to_native( @cgi.params['blog_name'][0] || '' )
 			title = @conf.to_native( @cgi.params['title'][0] || '' )
 			excerpt = @conf.to_native( @cgi.params['excerpt'][0] || '' )
+			if excerpt.length > 255
+				excerpt = @conf.shorten( excerpt, 252 )
+			end
 
 			body = [url, blog_name, title, excerpt].join("\n")
 			@cgi.params['name'] = ['TrackBack']
@@ -1792,10 +1741,12 @@ RSSFOOT
 			begin
 				@io.transaction( @date ) do |diaries|
 					@diaries = diaries
-					if @diaries[@date.strftime('%Y%m%d')].add_comment(@comment)
+					@diary = @diaries[@date.strftime('%Y%m%d')]
+					if @diary and comment_filter( @diary, @comment ) then
+						@diary.add_comment(@comment)
 						DIRTY_COMMENT
 					else
-						@error = "repeated TrackBack Ping"
+						@comment = nil
 						DIRTY_NONE
 					end
 				end
@@ -1806,7 +1757,8 @@ RSSFOOT
 
 		def eval_rhtml( prefix = '' )
 			raise TDiaryTrackBackError.new(@error) if @error
-			super
+			load_plugins
+			@plugin.instance_eval { update_proc }
 			TDiaryTrackBackBase::success_response
 		end
 	end
@@ -1819,7 +1771,11 @@ RSSFOOT
 		def eval_rhtml( prefix = '' )
 			load_plugins
 			anchor = @plugin.instance_eval(%Q|anchor "#{@date.strftime('%Y%m%d')}"|)
-			raise ForceRedirect::new("../#{@conf.index}#{anchor}#t")
+			url = "#{@conf.index}#{anchor}#t"
+			url[0, 0] = '../' if %r|^https?://|i !~ @conf.index
+			raise ForceRedirect::new( url )
 		end
 	end
 end
+
+# vim: ts=3
